<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Utilities\Util;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO\Response\Error;
use \Datetime;

class TransfMovimentacao extends CrudWithParameterBag {
    protected $parameterBag;
    protected $novoCodigo; 
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, Utilities\Util $util, Services\TransfMovimentacaoService $transferenciaMovimentacaoService, Services\NovoCodigo $novoCodigo, $crudService){
        $this->util = $util;
        $this->transferenciaMovimentacaoService = $transferenciaMovimentacaoService;
        $this->crudService = $crudService;
        $this->novoCodigo = $novoCodigo;
        parent::__construct($dataSourceManager, $parameterBag);
    }
    
    protected $dataSourceNameArray = array (
        'consumidor_origem' => 'vnd02900_transferenciamovimentacao_consumidor_origem',
        'consumidor_destino' => 'vnd02900_transferenciamovimentacao_consumidor_destino',
        'vendas'     => 'vnd02900_transferenciamovimentacao_vendas',
        'cliente'    => 'vnd02900_transferenciamovimentacao_cliente'
    );
    protected $parameterBagColumnsArray = array (
        'consumidor_origem' => array('CDCLIENTE', 'NRORG'),
        'consumidor_destino' => array('CDCLIENTE', 'NRORG'),
        'vendas'     => array('NRINSJURCLIE', 'CDCONSUMIDOR', 'DATAINI', 'DATAFIN'),
        'cliente'    => array('CDFILIAL', 'NRORG')
    );
    
    public function findConsumidorOrigem (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['consumidor_origem'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['consumidor_origem'];
        parent::find($request, $response);
    }
    
    public function findConsumidorDestino (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['consumidor_destino'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['consumidor_destino'];
        parent::find($request, $response);
    }
    
    public function findVendas (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['vendas'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['vendas'];
        parent::find($request, $response);
    }
    
    public function findCliente (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['cliente'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['cliente'];
        parent::find($request, $response);
    }
    
    private function deleteCrud(DTO\Request $request, DTO\Response $response) {
		try {
			$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $this->crudService->delete($response, $dataSet);
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
    
    public function transfereMovimentacao(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->transferenciaMovimentacaoService->alterSession();
        $cdsMoviClie = $this->adqMoviClie($row);
        $horaAtual = $this->getCurrentDate();
        $horaAtual = $horaAtual[0]['NOW'];
        $horaAtual = new DateTime(substr($horaAtual, 11));
        $novoCodigo;
         
        if(!empty($cdsMoviClie) || !$cdsMoviClie === null) {
            $cdsCentroCusto = $this->adqCentroCusto($row);
            
            for($i = 0; $i < count($cdsMoviClie); $i++) {
                $codigo = $this->geraNovoCodigo();
                $codigo = $codigo[0]['NOVOCODIGO'];
                $codigo = str_pad($codigo, 10, '0', STR_PAD_LEFT);
                $this->adqInsMov($row['CDCLIENTE_DESTINO'], $row['CDCONSUMIDOR_DESTINO'], $row['CDCLIENTE'], $row['CDCONSUMIDOR'], $codigo, $cdsMoviClie[$i], $row['NRORG']);
                $this->adqTranMovi($cdsMoviClie[$i], $row['CDCLIENTE_DESTINO'], $row['CDCONSUMIDOR_DESTINO'], $row['NRORG']);
                $this->adqTranVend($cdsMoviClie[$i], $row, $cdsCentroCusto[0]['CDCCUSCLIE']);

                $horaAtual->modify('+1 second');
                
                $this->adqInsereLog($row, $cdsMoviClie[$i]['DTMOVCLI'], $horaAtual->format('d/m/Y H:i:s'));
                $this->transferenciaMovimentacaoService->AtualizaSaldoCons($row['CDFILIAL'], $row['CDCLIENTE_DESTINO'], $row['CDCONSUMIDOR_DESTINO'], $cdsMoviClie[$i]['DTMOVCLI'], $row['CDOPERADOR']);
                $this->transferenciaMovimentacaoService->AtualizaSaldoCons($row['CDFILIAL'], $row['CDCLIENTE'], $row['CDCONSUMIDOR'], $cdsMoviClie[$i]['DTMOVCLI'], $row['CDOPERADOR']);
            }
            
            print "Transferência realizada com sucesso!0";
        } else {
            print "Não existe nenhuma movimentação para ser tranferida!1";
        }
    }
    
    public function adqMoviClie($row) {
        $params = array(
            ':CDCLIENTE'    => $row['CDCLIENTE'],
            ':CDCONSUMIDOR' => $row['CDCONSUMIDOR'],
            ':NRSEQVENDA'   => implode('_', $row['NRSEQVENDA']),
            ':DTMOVINI'     => $row['DATAINI'],
            ':DTMOVFIN'     => $row['DATAFIN'],
            ':NRORG'        => $row['NRORG']
        );
        
        $resultadoService = $this->transferenciaMovimentacaoService->adqMoviClie($params);
        return $resultadoService;
    }
    
    public function adqCentroCusto($row) {
        $params = array(
            ':CDCLIENTE' => $row['CDCLIENTE_DESTINO'],
            ':CDCONSUMIDOR' => $row['CDCONSUMIDOR_DESTINO'],
            ':NRORG'        => $row['NRORG']
        );
        
        $resultadoService = $this->transferenciaMovimentacaoService->adqCentroCusto($params);
        return $resultadoService;
    }
    
    public function adqInsMov($CDCLIENTE_DESTINO, $CDCONSUMIDOR_DESTINO, $CDCLIENTE, $CDCONSUMIDOR, $prxCodigo, $cdsMoviClie, $NRORG) {
        $params = array(
            ':CDCLIENTE'    => $CDCLIENTE_DESTINO,
            ':NRSEQMOVCLI'  => $prxCodigo,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR_DESTINO,
            ':VRMOVCLI'     => $cdsMoviClie['VRMOVCLI'],
            ':DTMOVCLI'     => $cdsMoviClie['DTMOVCLI'],
            ':DSHISTMOV'    => $cdsMoviClie['DSHISTMOV'],
            ':IDTIPMOCVLI'  => $cdsMoviClie['IDTIPMOCVLI'],
            ':CDFILIAL'     => $cdsMoviClie['CDFILIAL'],
            ':CDCAIXA'      => $cdsMoviClie['CDCAIXA'],
            ':NRSEQVENDA'   => $cdsMoviClie['NRSEQVENDA'],
            ':DTABERCAIX'   => $cdsMoviClie['DTABERCAIX'],
            ':NRSEQUMOVI'   => $cdsMoviClie['NRSEQUMOVI'],
            ':CDTIPORECE'   => $cdsMoviClie['CDTIPORECE'],
            ':VRSALDOCONS'  => $cdsMoviClie['VRSALDOCONS'],
            ':VRMOVPESCONS' => $cdsMoviClie['VRMOVPESCONS'],
            ':VRSALDPECONS' => $cdsMoviClie['VRSALDPECONS'],
            ':CDOPERADOR'   => $cdsMoviClie['CDOPERADOR'],
            ':CDOPERECSALD' => $cdsMoviClie['CDOPERECSALD'],
            ':DTATUMOVCLIE' => $cdsMoviClie['DTATUMOVCLIE'],
            ':DSATUMOVCLIE' => 'Transferência de Movimentação originada do Cliente '.$CDCLIENTE.' e do Consumidor '.$CDCONSUMIDOR,
            ':DTMOVIMCLIE'  => $cdsMoviClie['DTMOVIMCLIE'],
            ':DTRECMOVCLIE' => $cdsMoviClie['DTRECMOVCLIE'],
            ':CDFILIAPFATU' => $cdsMoviClie['CDFILIAPFATU'],
            ':NRAPURACFATU' => $cdsMoviClie['NRAPURACFATU'], 
            ':VRPESUBSIDIO' => $cdsMoviClie['VRPESUBSIDIO'],
            ':VRTETOSUBSID' => $cdsMoviClie['VRTETOSUBSID'],
            ':NRORG'        => $NRORG
        );
        
        $resultadoService = $this->transferenciaMovimentacaoService->adqInsMov($params);
        return $resultadoService;
    }
    
    public function adqTranMovi($cdsMoviClie, $CDCLIENTE_DESTINO, $CDCONSUMIDOR_DESTINO, $NRORG) {
        $params = array(
            ':DSATUMOVCLIE'    => 'Transferência de Movimentação para o Cliente '.$CDCLIENTE_DESTINO.' e Consumidor '.$CDCONSUMIDOR_DESTINO,
            ':OLD_CDCLIENTEDE' => $cdsMoviClie['CDCLIENTE'],
            ':OLD_NRSEQMOVCLI' => $cdsMoviClie['NRSEQMOVCLI'],
            ':NRORG' => $NRORG,
        );
        
        $resultadoService = $this->transferenciaMovimentacaoService->adqTranMovi($params);
        return $resultadoService;
    }
    
    public function adqTranVend($cdsMoviClie, $row, $CDCCUSCLIE) {
        $params = array(
            ':OLD_CDCLIENTEDE'    => $cdsMoviClie['CDCLIENTE'],
            ':OLD_CDCONSUMIDORDE' => $cdsMoviClie['CDCONSUMIDOR'],
            ':OLD_CDFILIAL'       => $cdsMoviClie['CDFILIAL'],
            ':OLD_NRSEQVENDA'     => $cdsMoviClie['NRSEQVENDA'],
            
            ':CDCLIENTE'    => $row['CDCLIENTE_DESTINO'],
            ':CDCONSUMIDOR' => $row['CDCONSUMIDOR_DESTINO'],
            ':TXOBSVENDA'   => 'Transferência de Movimentação originada do Cliente '.$cdsMoviClie['CDCLIENTE'].' e do Consumidor '.$cdsMoviClie['CDCONSUMIDOR'],
            ':CDCCUSCLIE'   => $CDCCUSCLIE,
            ':NRORG'        => $row['NRORG']
        );
        
        $resultadoService = $this->transferenciaMovimentacaoService->adqTranVend($params);
        return $resultadoService;
    }
    
    public function adqInsereLog($row, $DTMOVCLI, $dataAtual) {
        $params = array(
            ':CDOPERADOR'   => $row['CDOPERADOR'],
            ':DTTRMOVCONS'  => $dataAtual,
            ':CDCLIENDE'    => $row['CDCLIENTE'],
            ':CDCONSDE'     => $row['CDCONSUMIDOR'],
            ':CDFILIAL'     => $row['CDFILIAL'],
            ':CDCLIENPARA'  => $row['CDCLIENTE_DESTINO'],
            ':CDCONSPARA'   => $row['CDCONSUMIDOR_DESTINO'],
            ':DTINTRMOVCON' => $DTMOVCLI,
            ':NRORG'        => $row['NRORG']
        );
        
        $resultadoService = $this->transferenciaMovimentacaoService->adqInsereLog($params);
        return $resultadoService;
    }
    
    private function geraNovoCodigo() {
        $resultadoService = $this->transferenciaMovimentacaoService->geraNovoCodigo();
        return $resultadoService;
    }
    
    private function getCurrentDate() {
        $resultadoService = $this->transferenciaMovimentacaoService->getCurrentDate();
        return $resultadoService;
    }
}